from .geolocation import Geolocation


class Dir:
    NOWAY = 10
    UNKNOWN = 1
    APPROACHING = 11
    RECEDING = 12

    def __init__(self, direction):
        """
        :type direction: int
        """
        self._int_to_str_map = {
            Dir.UNKNOWN: 'unknown',
            Dir.APPROACHING: 'approaching',
            Dir.RECEDING: 'receding',
            Dir.NOWAY: 'any'
        }
        self.dir = direction

    @staticmethod
    def deserialize(direction):
        """
        :type direction: str
        :rtype: Dir
        """
        str_to_int_map = {
            'unknown': Dir.UNKNOWN,
            'approaching': Dir.APPROACHING,
            'receding': Dir.RECEDING,
            'any': Dir.NOWAY
        }
        int_dir = str_to_int_map.get(direction)

        if int_dir is None:
            raise ValueError

        return Dir(int_dir)

    def __eq__(self, other):
        """
        :type other: Dir
        :rtype: bool
        """
        return self.dir == other.dir

    def __ne__(self, other):
        """
        :type other: Dir
        :rtype: bool
        """
        return self.dir != other.dir

    def __str__(self):
        """
        :rtype: str
        """
        return self._int_to_str_map[self.dir]


class Location:
    def __init__(self, place='', lane=''):
        """
        :type place: str
        :type lane: str
        """
        self.place = place
        self.lane = lane

    @staticmethod
    def deserialize(jnode):
        """
        :type jnode: dict
        :rtype: Location
        """
        place = jnode.get('place', '')
        return Location(place)

    def serialize(self, parent_jnode, jnode_key='location'):
        """
        :type parent_jnode: dict
        :type jnode_key: str
        :rtype: dict
        """
        jnode = {
        }

        if len(self.place):
            jnode['place'] = self.place

        if len(self.lane):
            jnode['lane'] = self.lane

        if len(self.place) or len(self.lane):
            parent_jnode[jnode_key] = jnode

        return parent_jnode


class Origin:
    def __init__(self, uuid, time, location, geolocation=None):
        """
        :type uuid: str
        :type time: str
        :type location: Location
        :type geolocation: Geolocation | None
        """
        self._location = location
        self._geolocation = geolocation

        self._jnode = {
            'uuid': uuid,
            'time': time
        }

    @property
    def uuid(self):
        """
        :rtype: str
        """
        return self._jnode['uuid']

    @property
    def time(self):
        """
        :rtype: str
        """
        return self._jnode['time']

    @property
    def location(self):
        """
        :rtype: Location
        """
        return self._location

    @property
    def geolocation(self):
        """
        :rtype: Geolocation | None
        """
        return self._geolocation

    def serialize(self):
        """
        :rtype: dict
        """
        self._location.serialize(self._jnode)

        if not self._jnode.get('geolocation') and self._geolocation:
            self._jnode['geolocation'] = self._geolocation.serialize_dict()

        return self._jnode
